--[[
This is free and unencumbered software released into the public domain.

Anyone is free to copy, modify, publish, use, compile, sell, or
distribute this software, either in source code form or as a compiled
binary, for any purpose, commercial or non-commercial, and by any
means.

See <LICENSE.txt> for the whole license.
For more information, see <http://unlicense.org>.
]]

local prodModuleList = {"productivity-module","productivity-module-2","productivity-module-3"}

--[[
	Remove an entity from the game.
	Since we don't want user to be able to give themself those entity, we remove
	them completly from the game instead of disabling them.
]]
factorioEducationalMod.removeEntity = function(entityName, entityType)
	if(data.raw[entityType] and data.raw[entityType][entityName]) then
		data.raw[entityType][entityName] = nil
	end
end

--[[
	Remove a recipe from the game.
]]
factorioEducationalMod.removeRecipe = function(recipeName)
	if(data.raw.recipe[recipeName]) then
		data.raw.recipe[recipeName] = nil
	end
end

--[[
	Remove an item from the game.
]]
factorioEducationalMod.removeItem = function(itemName)
	if(data.raw.item[itemName]) then
		data.raw.item[itemName] = nil
	end
end


--[[
	Remove an entity, its item and its recipe.
	If no recipe name is given, it will try to remove the recipe named like the entity name.
]]
factorioEducationalMod.removeRecipeEntityAndItem = function(entityAndItemName,entityType,recipeName)
	recipeName = recipeName or entityAndItemName
	factorioEducationalMod.removeEntity(entityAndItemName,entityType)
	factorioEducationalMod.removeItem(entityAndItemName)
	factorioEducationalMod.removeRecipe(recipeName)
end

--[[
	Remove an item and its recipe.
	If no recipe name is given, it will try to remove the recipe named like the entity name.
]]
factorioEducationalMod.removeRecipeAndItem = function(entityAndItemName,entityType,recipeName)
	recipeName = recipeName or entityAndItemName
	factorioEducationalMod.removeItem(entityAndItemName)
	factorioEducationalMod.removeRecipe(recipeName)
end

--[[
	Remove an entity and its recipe.
	If no recipe name is given, it will try to remove the recipe named like the entity name.
]]
factorioEducationalMod.removeRecipeAndEntity = function(entityName,entityType,recipeName)
	recipeName = recipeName or entityName
	factorioEducationalMod.removeEntity(entityName,entityType)
	factorioEducationalMod.removeRecipe(recipeName)
end

--[[
	Remove a technology.
]]
factorioEducationalMod.removeTechnology = function(technologyName)
	if(data.raw.technology[technologyName]) then
		data.raw.technology[technologyName] = nil
	end
end

--[[
	Remove all technology that begin with the given name and is followed by a number.
	Will stop when no other thechnology is found.
	The fisrt number checked can be given if the technology don't begin at 1.
]]
factorioEducationalMod.removeTechnologyIncrement = function(technologyName,firstNumber)
	local i = firstNumber or 1
	while (data.raw.technology[technologyName..i]) do
		data.raw.technology[technologyName..i] = nil
		i=i+1
	end
end

--[[
	Remove a prerequisite from a technology.
	Only accept string, call multiple time for each prerequisite to remove.
]]
factorioEducationalMod.removeTechnologyPrerequisite = function(technologyNameToRemoveFrom,prerequisiteToRemove)
	-- check tech existence
	local tech = data.raw.technology[technologyNameToRemoveFrom] or nil
	if(not data.raw.technology[technologyNameToRemoveFrom]) then
		return
	end
	-- building prerequisites table without the one we want to remove
	local newPrerequisites = {}
	for _,prereq in pairs(tech.prerequisites) do
		if (prereq ~= prerequisiteToRemove) then
			table.insert(newPrerequisites,prereq)
		end
	end
	-- apply new table to actual prerequisites
	tech.prerequisites = newPrerequisites
end

--[[
	Add new prerequisites to a technology.
	Accept a string or a table of string.
	Old prerequisites are kept. Remove them first if you don't want them.
]]
factorioEducationalMod.addTechnologyPrerequisite = function(technologyNameToAddTo,prerequisitesToAdd)
	-- check tech existence
	local tech = data.raw.technology[technologyNameToAddTo] or nil
	if(not data.raw.technology[technologyNameToAddTo]) then
		return
	end
	if(type(prerequisitesToAdd)== "string") then
		prerequisitesToAdd = {prerequisitesToAdd}
	end
	for _,prereq in pairs(prerequisitesToAdd) do
		table.insert(tech.prerequisites,prereq)
	end
end

--[[
	Remove a technology recipe unlock.
	Only accept string, call multiple time for each recipe to remove.
]]
factorioEducationalMod.removeTechnologyRecipeUnlock = function(technologyNameToRemoveFrom,recipeToRemove)
	-- check tech existence
	local tech = data.raw.technology[technologyNameToRemoveFrom] or nil
	if(not data.raw.technology[technologyNameToRemoveFrom]) then
		return
	end
	-- building effects table without the one we want to remove
	local newEffects = {}
	for _,effect in pairs(tech.effects) do
		if (not effect.recipe or effect.recipe ~= recipeToRemove) then
			table.insert(newEffects,effect)
		end
	end
	-- apply new table to actual prerequisites
	tech.effects = newEffects
end

--[[
	Add new recipe unlock effect to a technology.
	Only accept string, call multiple time for each recipe to add.
]]
factorioEducationalMod.addTechnologyRecipeUnlock = function(technologyNameToAddTo,recipeNameToAdd)
	-- check tech existence
	local tech = data.raw.technology[technologyNameToAddTo] or nil
	if(not data.raw.technology[technologyNameToAddTo]) then
		return
	end
	table.insert(tech.effects,{recipe=recipeNameToAdd,type="unlock-recipe"})
end


--[[
	Remove an item from productivity modules list limitation.
	Only accept string, call multiple time for each item to remove.
]]
factorioEducationalMod.removeItemFromProdModuleLimitation = function(itemToRemove)
	for _,module in pairs(prodModuleList) do
		local newLimitation = {}
		for _,item in pairs(data.raw.module[module].limitation) do
			if (item ~= itemToRemove) then
				table.insert(newLimitation,item)
			end
		end
		-- apply new table to actual prerequisites
		data.raw.module[module].limitation = newLimitation
	end
end
